@@ -36,6 +36,8 @@
 #include "asb-package-rpm.h"
 #endif
 
+#undef HAVE_FONTS
+
 static gchar *
 asb_test_get_filename (const gchar *filename)
 {
@@ -240,6 +242,20 @@ asb_test_package_func (void)
 	asb_package_set_release (pkg2, "2.fc22");
 }
 
+static void
+asb_test_package_guess_from_fn_func (void)
+{
+	g_autoptr(AsbPackage) pkg = asb_package_new ();
+
+	/* check pathological name */
+	asb_package_set_filename (pkg, "/tmp/atom.x86_64.rpm");
+	g_assert_cmpstr (asb_package_get_name (pkg), ==, NULL);
+	g_assert_cmpstr (asb_package_get_version (pkg), ==, NULL);
+	g_assert_cmpstr (asb_package_get_release_str (pkg), ==, NULL);
+	g_assert_cmpstr (asb_package_get_arch (pkg), ==, "x86_64");
+	g_assert_cmpint (asb_package_get_epoch (pkg), ==, 0);
+}
+
 static void
 asb_test_utils_glob_func (void)
 {
@@ -321,8 +337,11 @@ asb_test_context_func (void)
 		"app-console-1-1.fc25.noarch.rpm",	/* app with no icon */
 		"app-1-1.fc25.i686.rpm",		/* GUI multiarch app */
 		"composite-1-1.fc21.x86_64.rpm",	/* multiple GUI apps */
+		"driver-1-1.fc27.noarch.rpm",		/* a hardware driver */
+#ifdef HAVE_FONTS
 		"font-1-1.fc21.noarch.rpm",		/* font */
 		"font-serif-1-1.fc21.noarch.rpm",	/* font that extends */
+#endif
 		"colorhug-als-2.0.2.cab",		/* firmware */
 		NULL};
 
@@ -369,7 +388,11 @@ asb_test_context_func (void)
 	}
 
 	/* verify queue size */
-	g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 9);
+#ifdef HAVE_FONTS
+	g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 10);
+#else
+	g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 8);
+#endif
 
 	/* run the plugins */
 	ret = asb_context_process (ctx, &error);
@@ -381,7 +404,9 @@ asb_test_context_func (void)
 	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-failed.xml.gz", G_FILE_TEST_EXISTS));
 	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-ignore.xml.gz", G_FILE_TEST_EXISTS));
 	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-icons.tar.gz", G_FILE_TEST_EXISTS));
+#ifdef HAVE_FONTS
 	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-screenshots.tar", G_FILE_TEST_EXISTS));
+#endif
 
 	/* load AppStream metadata */
 	file = g_file_new_for_path ("/tmp/asbuilder/output/appstream.xml.gz");
@@ -390,9 +415,9 @@ asb_test_context_func (void)
 	g_assert_no_error (error);
 	g_assert (ret);
 #ifdef HAVE_FONTS
-//	g_assert_cmpint (as_store_get_size (store), ==, 5);
+//	g_assert_cmpint (as_store_get_size (store), ==, 6);
 #else
-	g_assert_cmpint (as_store_get_size (store), ==, 4);
+	g_assert_cmpint (as_store_get_size (store), ==, 5);
 #endif
 	app = as_store_get_app_by_pkgname (store, "app");
 	g_assert (app != NULL);
@@ -491,6 +516,7 @@ asb_test_context_func (void)
 		"<provides>\n"
 		"<dbus type=\"session\">org.freedesktop.AppStream</dbus>\n"
 		"</provides>\n"
+		"<launchable type=\"desktop-id\">app.desktop</launchable>\n"
 		"<languages>\n"
 		"<lang percentage=\"100\">en_GB</lang>\n"
 		"<lang percentage=\"33\">ru</lang>\n"
@@ -524,6 +550,18 @@ asb_test_context_func (void)
 		"</provides>\n"
 		"</component>\n"
 #endif
+		"<component type=\"driver\">\n"
+		"<id>driver</id>\n"
+		"<pkgname>driver</pkgname>\n"
+		"<name>Driver</name>\n"
+		"<summary>A hardware driver</summary>\n"
+		"<icon type=\"cached\" height=\"64\" width=\"64\">driver.png</icon>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
+		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
+		"<releases>\n"
+		"<release timestamp=\"1517313600\" version=\"1\"/>\n"
+		"</releases>\n"
+		"</component>\n"
 		"</components>\n";
 	ret = asb_test_compare_lines (xml->str, expected_xml, &error);
 	g_assert_no_error (error);
@@ -535,7 +573,9 @@ asb_test_context_func (void)
 	ret = as_store_from_file (store_failed, file_failed, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
+#ifdef HAVE_FONTS
 	g_assert_cmpint (as_store_get_size (store_failed), ==, 1);
+#endif
 //	app = as_store_get_app_by_id (store_failed, "console1.desktop");
 //	g_assert (app != NULL);
 //	app = as_store_get_app_by_id (store_failed, "console2.desktop");
@@ -543,9 +583,9 @@ asb_test_context_func (void)
 
 	/* check output */
 	xml_failed = as_store_to_xml (store_failed, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
+#ifdef HAVE_FONTS
 	expected_xml =
 		"<components origin=\"asb-self-test-failed\" version=\"0.9\">\n"
-#ifdef HAVE_FONTS
 		"<component type=\"font\">\n"
 		"<id>LiberationSerif</id>\n"
 		"<pkgname>font-serif</pkgname>\n"
@@ -580,8 +620,11 @@ asb_test_context_func (void)
 		"<lang>en</lang>\n"
 		"</languages>\n"
 		"</component>\n"
-#endif
 		"</components>\n";
+#else
+	expected_xml =
+		"<components origin=\"asb-self-test-failed\" version=\"0.9\"/>\n";
+#endif
 	ret = asb_test_compare_lines (xml_failed->str, expected_xml, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
@@ -609,10 +652,12 @@ asb_test_context_func (void)
 		"<id>composite.x86_64</id>\n"
 		"<pkgname>composite</pkgname>\n"
 		"</component>\n"
+#ifdef HAVE_FONTS
 		"<component type=\"generic\">\n"
 		"<id>font-serif.noarch</id>\n"
 		"<pkgname>font-serif</pkgname>\n"
 		"</component>\n"
+#endif
 		"<component type=\"generic\">\n"
 		"<id>test.noarch</id>\n"
 		"<pkgname>test</pkgname>\n"
@@ -760,9 +805,11 @@ main (int argc, char **argv)
 	/* only critical and error are fatal */
 	g_log_set_fatal_mask (NULL, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
 	g_setenv ("ASB_IS_SELF_TEST", "", TRUE);
+	g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
 
 	/* tests go here */
 	g_test_add_func ("/AppStreamBuilder/package", asb_test_package_func);
+	g_test_add_func ("/AppStreamBuilder/package{guess-fn}", asb_test_package_guess_from_fn_func);
 	g_test_add_func ("/AppStreamBuilder/utils{glob}", asb_test_utils_glob_func);
 	g_test_add_func ("/AppStreamBuilder/plugin-loader", asb_test_plugin_loader_func);
 	g_test_add_func ("/AppStreamBuilder/firmware", asb_test_firmware_func);
