@@ -97,6 +97,8 @@ as_checksum_target_from_string (const gchar *target)
 		return AS_CHECKSUM_TARGET_CONTAINER;
 	if (g_strcmp0 (target, "content") == 0)
 		return AS_CHECKSUM_TARGET_CONTENT;
+	if (g_strcmp0 (target, "signature") == 0)
+		return AS_CHECKSUM_TARGET_SIGNATURE;
 	return AS_CHECKSUM_TARGET_UNKNOWN;
 }
 
@@ -117,6 +119,8 @@ as_checksum_target_to_string (AsChecksumTarget target)
 		return "container";
 	if (target == AS_CHECKSUM_TARGET_CONTENT)
 		return "content";
+	if (target == AS_CHECKSUM_TARGET_SIGNATURE)
+		return "signature";
 	return NULL;
 }
 
@@ -341,8 +345,8 @@ as_checksum_node_parse (AsChecksum *checksum, GNode *node,
 	tmp = as_node_get_attribute (node, "target");
 	if (tmp != NULL)
 		priv->target = as_checksum_target_from_string (tmp);
-	as_ref_string_assign (&priv->filename, as_node_get_attribute (node, "filename"));
-	as_ref_string_assign (&priv->value, as_node_get_data (node));
+	as_ref_string_assign (&priv->filename, as_node_get_attribute_as_refstr (node, "filename"));
+	as_ref_string_assign (&priv->value, as_node_get_data_as_refstr (node));
 	return TRUE;
 }
 
