@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2014-2017 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -48,8 +48,10 @@ typedef struct
 	AsRefString		*filename;
 	AsRefString		*prefix;
 	AsRefString		*prefix_private;
+	gboolean		 prepend_size;
 	guint			 width;
 	guint			 height;
+	guint			 scale;
 	GdkPixbuf		*pixbuf;
 	GBytes			*data;
 } AsIconPrivate;
@@ -223,6 +225,25 @@ const gchar *
 as_icon_get_prefix (AsIcon *icon)
 {
 	AsIconPrivate *priv = GET_PRIVATE (icon);
+
+	/* only use the size if the metadata has width and height */
+	if (priv->prepend_size && priv->prefix_private == NULL) {
+		g_autofree gchar *sz = NULL;
+		if (priv->scale > 1) {
+			sz = g_strdup_printf ("%s/%ux%u@%u",
+					      priv->prefix,
+					      priv->width,
+					      priv->height,
+					      priv->scale);
+		} else {
+			sz = g_strdup_printf ("%s/%ux%u",
+					      priv->prefix,
+					      priv->width,
+					      priv->height);
+		}
+		as_ref_string_assign_safe (&priv->prefix_private, sz);
+	}
+
 	if (priv->prefix_private != NULL)
 		return priv->prefix_private;
 	return priv->prefix;
@@ -262,6 +283,23 @@ as_icon_get_height (AsIcon *icon)
 	return priv->height;
 }
 
+/**
+ * as_icon_get_scale:
+ * @icon: a #AsIcon instance.
+ *
+ * Gets the icon scale.
+ *
+ * Returns: scale factor
+ *
+ * Since: 0.6.13
+ **/
+guint
+as_icon_get_scale (AsIcon *icon)
+{
+	AsIconPrivate *priv = GET_PRIVATE (icon);
+	return priv->scale;
+}
+
 /**
  * as_icon_get_kind:
  * @icon: a #AsIcon instance.
@@ -345,6 +383,13 @@ as_icon_set_prefix (AsIcon *icon, const gchar *prefix)
 	as_ref_string_assign_safe (&priv->prefix, prefix);
 }
 
+void
+as_icon_set_prefix_rstr (AsIcon *icon, AsRefString *rstr)
+{
+	AsIconPrivate *priv = GET_PRIVATE (icon);
+	as_ref_string_assign (&priv->prefix, rstr);
+}
+
 /**
  * as_icon_set_url:
  * @icon: a #AsIcon instance.
@@ -409,6 +454,22 @@ as_icon_set_height (AsIcon *icon, guint height)
 	priv->height = height;
 }
 
+/**
+ * as_icon_set_scale:
+ * @icon: a #AsIcon instance.
+ * @scale: the scale as a factor.
+ *
+ * Sets the icon scale.
+ *
+ * Since: 0.6.13
+ **/
+void
+as_icon_set_scale (AsIcon *icon, guint scale)
+{
+	AsIconPrivate *priv = GET_PRIVATE (icon);
+	priv->scale = scale;
+}
+
 /**
  * as_icon_set_kind:
  * @icon: a #AsIcon instance.
@@ -479,10 +540,10 @@ as_icon_node_insert_embedded (AsIcon *icon, GNode *parent, AsNodeContext *ctx)
 	n = as_node_insert (parent, "icon", NULL, 0,
 			    "type", as_icon_kind_to_string (priv->kind),
 			    NULL);
-	if (as_node_context_get_version (ctx) >= 0.8) {
-		as_node_add_attribute_as_uint (n, "width", priv->width);
-		as_node_add_attribute_as_uint (n, "height", priv->height);
-	}
+	as_node_add_attribute_as_uint (n, "width", priv->width);
+	as_node_add_attribute_as_uint (n, "height", priv->height);
+	if (priv->scale > 1)
+		as_node_add_attribute_as_uint (n, "scale", priv->scale);
 	as_node_insert (n, "name", priv->name, 0, NULL);
 	data = g_base64_encode (g_bytes_get_data (priv->data, NULL),
 				g_bytes_get_size (priv->data));
@@ -539,12 +600,13 @@ as_icon_node_insert (AsIcon *icon, GNode *parent, AsNodeContext *ctx)
 		}
 		break;
 	}
-	if (priv->kind == AS_ICON_KIND_CACHED &&
-	    as_node_context_get_version (ctx) >= 0.8) {
+	if (priv->kind == AS_ICON_KIND_CACHED) {
 		if (priv->width > 0)
 			as_node_add_attribute_as_uint (n, "width", priv->width);
 		if (priv->height > 0)
 			as_node_add_attribute_as_uint (n, "height", priv->height);
+		if (priv->scale > 1)
+			as_node_add_attribute_as_uint (n, "scale", priv->scale);
 	}
 	return n;
 }
@@ -621,12 +683,11 @@ as_icon_node_parse (AsIcon *icon, GNode *node,
 		    AsNodeContext *ctx, GError **error)
 {
 	AsIconPrivate *priv = GET_PRIVATE (icon);
-	const gchar *tmp;
+	AsRefString *str;
 	guint size;
-	gboolean prepend_size = TRUE;
 
-	tmp = as_node_get_attribute (node, "type");
-	as_icon_set_kind (icon, as_icon_kind_from_string (tmp));
+	str = as_node_get_attribute_as_refstr (node, "type");
+	as_icon_set_kind (icon, as_icon_kind_from_string (str));
 	switch (priv->kind) {
 	case AS_ICON_KIND_EMBEDDED:
 		if (!as_icon_node_parse_embedded (icon, node, error))
@@ -635,8 +696,8 @@ as_icon_node_parse (AsIcon *icon, GNode *node,
 	default:
 
 		/* preserve the URL for remote icons */
-		tmp = as_node_get_data (node);
-		if (tmp == NULL) {
+		str = as_node_get_data_as_refstr (node);
+		if (str == NULL) {
 			g_set_error (error,
 				     AS_ICON_ERROR,
 				     AS_ICON_ERROR_FAILED,
@@ -645,24 +706,25 @@ as_icon_node_parse (AsIcon *icon, GNode *node,
 			return FALSE;
 		}
 		if (priv->kind == AS_ICON_KIND_REMOTE)
-			as_icon_set_url (icon, tmp);
+			as_ref_string_assign (&priv->url, str);
 		else if (priv->kind == AS_ICON_KIND_LOCAL)
-			as_icon_set_filename (icon, tmp);
+			as_ref_string_assign (&priv->filename, str);
 
 		/* store the name without any prefix */
-		if (g_strstr_len (tmp, -1, "/") == NULL) {
-			as_icon_set_name (icon, tmp);
+		if (g_strstr_len (str, -1, "/") == NULL) {
+			as_ref_string_assign (&priv->name, str);
 		} else {
 			g_autofree gchar *basename = NULL;
-			basename = g_path_get_basename (tmp);
+			basename = g_path_get_basename (str);
 			as_icon_set_name (icon, basename);
 		}
 
 		/* width is optional, assume 64px if missing */
+		priv->prepend_size = TRUE;
 		size = as_node_get_attribute_as_uint (node, "width");
 		if (size == G_MAXUINT) {
 			size = 64;
-			prepend_size = FALSE;
+			priv->prepend_size = FALSE;
 		}
 		priv->width = size;
 
@@ -670,18 +732,15 @@ as_icon_node_parse (AsIcon *icon, GNode *node,
 		size = as_node_get_attribute_as_uint (node, "height");
 		if (size == G_MAXUINT) {
 			size = 64;
-			prepend_size = FALSE;
+			priv->prepend_size = FALSE;
 		}
 		priv->height = size;
 
-		/* only use the size if the metadata has width and height */
-		if (prepend_size) {
-			g_autofree gchar *sz = g_strdup_printf ("%s/%ux%u",
-								priv->prefix,
-								priv->width,
-								priv->height);
-			as_ref_string_assign_safe (&priv->prefix_private, sz);
-		}
+		/* scale is optional, assume 1 if missing */
+		size = as_node_get_attribute_as_uint (node, "scale");
+		if (size == G_MAXUINT)
+			size = 1;
+		priv->scale = size;
 		break;
 	}
 
@@ -723,6 +782,11 @@ as_icon_node_parse_dep11 (AsIcon *icon, GNode *node,
 			if (size == G_MAXUINT)
 				size = 64;
 			priv->height = size;
+		} else if (g_strcmp0 (key, "scale") == 0) {
+			size = as_yaml_node_get_value_as_uint (n);
+			if (size == G_MAXUINT)
+				size = 1;
+			priv->scale = size;
 		} else {
 			if (priv->kind == AS_ICON_KIND_REMOTE) {
 				if (g_strcmp0 (key, "url") == 0) {
