@@ -469,6 +469,7 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node,
 			  AsNodeContext *ctx, GError **error)
 {
 	AsScreenshotPrivate *priv = GET_PRIVATE (screenshot);
+	AsRefString *str;
 	GList *l;
 	GNode *c;
 	const gchar *tmp;
@@ -489,18 +490,20 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node,
 	captions = as_node_get_localized (node, "caption");
 	if (captions != NULL) {
 		g_autoptr(GList) keys = NULL;
+		as_screenshot_ensure_captions (screenshot);
 		keys = g_hash_table_get_keys (captions);
 		for (l = keys; l != NULL; l = l->next) {
-			tmp = l->data;
-			as_screenshot_set_caption (screenshot,
-						   tmp,
-						   g_hash_table_lookup (captions, tmp));
+			AsRefString *locale = l->data;
+			AsRefString *caption = g_hash_table_lookup (captions, locale);
+			g_hash_table_insert (priv->captions,
+					     as_ref_string_ref (locale),
+					     as_ref_string_ref (caption));
 		}
 	}
 
 	/* AppData files does not have <image> tags */
-	tmp = as_node_get_data (node);
-	if (tmp != NULL) {
+	str = as_node_get_data_as_refstr (node);
+	if (str != NULL) {
 		AsImage *image;
 		image = as_image_new ();
 		as_image_set_kind (image, AS_IMAGE_KIND_SOURCE);
@@ -510,7 +513,7 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node,
 		size = as_node_get_attribute_as_uint (node, "height");
 		if (size != G_MAXUINT)
 			as_image_set_height (image, size);
-		as_image_set_url (image, tmp);
+		as_image_set_url_rstr (image, str);
 		g_ptr_array_add (priv->images, image);
 	}
 
